// Copyright 2025 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package main

import (
	"bytes"
	"log"
	"os"
)

func main() {
	data, err := os.ReadFile("big.test.old")
	if err != nil {
		log.Fatal(err)
	}

	cut := []byte{
		0x48, 0x8b, 0x84, 0x24, 0xb8, 0x00, 0x00, 0x00, // MOVQ 0xb8(SP), AX
		0x44, 0x88, 0x84, 0x24, 0x80, 0x00, 0x00, 0x00, // MOVB R8, 0x80(SP)
		0x4c, 0x8b, 0x84, 0x24, 0xb0, 0x00, 0x00, 0x00, // MOVQ 0xb0(SP), R8
		0x4c, 0x89, 0x84, 0x24, 0xb8, 0x00, 0x00, 0x00, // MOVQ R8, 0xb8(SP)
		0x48, 0x89, 0x84, 0x24, 0xb0, 0x00, 0x00, 0x00, // MOVQ AX, 0xb0(SP)
		0x48, 0x8b, 0x84, 0x24, 0x18, 0x01, 0x00, 0x00, // MOVQ 0x118(SP), AX
		0x44, 0x0f, 0xb6, 0x84, 0x24, 0x80, 0x00, 0x00, 0x00, // MOVZX 0x80(SP), R8
	}
	n := bytes.Count(data, cut)
	if n > 1 {
		log.Fatal("too many cut patterns")
	}
	if n == 0 {
		log.Fatal("did not find cut pattern")
	}
	i := bytes.Index(data, cut)
	data[i] = 0xeb
	data[i+1] = byte(len(cut) - 2)

	fixOld := []byte{
		0x4c, 0x89, 0x84, 0x24, 0xb8, 0x00, 0x00, 0x00, // MOVQ R8, 0xb8(SP)
		0x4c, 0x89, 0xac, 0x24, 0xd0, 0x00, 0x00, 0x00, // MOVQ R13, 0xd0(SP)
		0x48, 0x89, 0xb4, 0x24, 0xb0, 0x00, 0x00, 0x00, // MOVQ SI, 0xb0(SP)
	}
	fixNew := []byte{
		0x4c, 0x89, 0x84, 0x24, 0xb0, 0x00, 0x00, 0x00, // MOVQ R8, 0xb0(SP)
		0x4c, 0x89, 0xac, 0x24, 0xd0, 0x00, 0x00, 0x00, // MOVQ R13, 0xd0(SP)
		0x48, 0x89, 0xb4, 0x24, 0xb8, 0x00, 0x00, 0x00, // MOVQ SI, 0xb8(SP)
	}
	n = bytes.Count(data, fixOld)
	if n > 1 {
		log.Fatal("too many fixOld patterns")
	}
	if n == 0 {
		log.Fatal("did not find fixOld pattern")
	}
	i = bytes.Index(data, fixOld)
	copy(data[i:], fixNew)

	if err := os.WriteFile("big.test.old.fix", data, 0666); err != nil {
		log.Fatal(err)
	}
}
